/*
 * Name:    libMp.c
 *
 * Purpose: This file contains miscellaneous Python functions that are
 *          used by multiple MP-based Python modules.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2009 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_handle.h"
#include "mp.h"

/* MpError_SetString() is a varargs version of PyErr_SetString. */

MX_EXPORT void
MpError_SetString( PyObject *exception_object, char *format, ... )
{
	va_list args;
	char buffer[2500];

	va_start(args, format);
	vsnprintf(buffer, sizeof(buffer), format, args);
	va_end(args);

	PyErr_SetString( exception_object, buffer );

	return;
}

/* These are the exception objects for MP. */

MX_EXPORT PyObject *MpPrivateExc_Error = NULL;

MX_EXPORT PyObject *MpPrivateExc_Success = NULL;
MX_EXPORT PyObject *MpPrivateExc_Null_Argument = NULL;
MX_EXPORT PyObject *MpPrivateExc_Illegal_Argument = NULL;
MX_EXPORT PyObject *MpPrivateExc_Corrupt_Data_Structure = NULL;
MX_EXPORT PyObject *MpPrivateExc_Unparseable_String = NULL;
MX_EXPORT PyObject *MpPrivateExc_End_Of_Data = NULL;
MX_EXPORT PyObject *MpPrivateExc_Unexpected_End_Of_Data = NULL;
MX_EXPORT PyObject *MpPrivateExc_Not_Found = NULL;
MX_EXPORT PyObject *MpPrivateExc_Type_Mismatch = NULL;
MX_EXPORT PyObject *MpPrivateExc_Not_Yet_Implemented = NULL;
MX_EXPORT PyObject *MpPrivateExc_Unsupported = NULL;
MX_EXPORT PyObject *MpPrivateExc_Out_Of_Memory = NULL;
MX_EXPORT PyObject *MpPrivateExc_Would_Exceed_Limit = NULL;
MX_EXPORT PyObject *MpPrivateExc_Limit_Was_Exceeded = NULL;
MX_EXPORT PyObject *MpPrivateExc_Interface_IO_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Device_IO_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_File_IO_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Terminal_IO_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_IPC_IO_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_IPC_Connection_Lost = NULL;
MX_EXPORT PyObject *MpPrivateExc_Network_IO_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Network_Connection_Lost = NULL;
MX_EXPORT PyObject *MpPrivateExc_Not_Ready = NULL;
MX_EXPORT PyObject *MpPrivateExc_Interrupted = NULL;
MX_EXPORT PyObject *MpPrivateExc_Pause_Requested = NULL;
MX_EXPORT PyObject *MpPrivateExc_Interface_Action_Failed = NULL;
MX_EXPORT PyObject *MpPrivateExc_Device_Action_Failed = NULL;
MX_EXPORT PyObject *MpPrivateExc_Function_Failed = NULL;
MX_EXPORT PyObject *MpPrivateExc_Controller_Internal_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Permission_Denied = NULL;
MX_EXPORT PyObject *MpPrivateExc_Client_Request_Denied = NULL;
MX_EXPORT PyObject *MpPrivateExc_Try_Again = NULL;
MX_EXPORT PyObject *MpPrivateExc_Timed_Out = NULL;
MX_EXPORT PyObject *MpPrivateExc_Hardware_Configuration_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Hardware_Fault = NULL;
MX_EXPORT PyObject *MpPrivateExc_Record_Disabled_Due_To_Fault = NULL;
MX_EXPORT PyObject *MpPrivateExc_Record_Disabled_By_User = NULL;
MX_EXPORT PyObject *MpPrivateExc_Initialization_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Read_Only = NULL;
MX_EXPORT PyObject *MpPrivateExc_Software_Configuration_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Operating_System_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Unknown_Error = NULL;
MX_EXPORT PyObject *MpPrivateExc_Not_Valid_For_Current_State = NULL;
MX_EXPORT PyObject *MpPrivateExc_Configuration_Conflict = NULL;
MX_EXPORT PyObject *MpPrivateExc_Not_Available = NULL;
MX_EXPORT PyObject *MpPrivateExc_Stop_Requested = NULL;
MX_EXPORT PyObject *MpPrivateExc_Bad_Handle = NULL;
MX_EXPORT PyObject *MpPrivateExc_Object_Abandoned = NULL;
MX_EXPORT PyObject *MpPrivateExc_Might_Cause_Deadlock = NULL;
MX_EXPORT PyObject *MpPrivateExc_Already_Exists = NULL;
MX_EXPORT PyObject *MpPrivateExc_Invalid_Callback = NULL;
MX_EXPORT PyObject *MpPrivateExc_Early_Exit = NULL;

/*---*/

static struct {
	PyObject **exception_object_address;
	char exception_name[40];
} MpPrivate_exception_table[] = {

{ &MpPrivateExc_Success,                      "Success_Error" },
{ &MpPrivateExc_Null_Argument,                "Null_Argument_Error" },
{ &MpPrivateExc_Illegal_Argument,             "Illegal_Argument_Error" },
{ &MpPrivateExc_Corrupt_Data_Structure,       "Corrupt_Data_Structure_Error" },
{ &MpPrivateExc_Unparseable_String,           "Unparseable_String_Error" },
{ &MpPrivateExc_End_Of_Data,                  "End_Of_Data_Error" },
{ &MpPrivateExc_Unexpected_End_Of_Data,       "Unexpected_End_Of_Data_Error" },
{ &MpPrivateExc_Not_Found,                    "Not_Found_Error" },
{ &MpPrivateExc_Type_Mismatch,                "Type_Mismatch_Error" },
{ &MpPrivateExc_Not_Yet_Implemented,          "Not_Yet_Implemented_Error" },
{ &MpPrivateExc_Unsupported,                  "Unsupported_Error" },
{ &MpPrivateExc_Out_Of_Memory,                "Out_Of_Memory_Error" },
{ &MpPrivateExc_Would_Exceed_Limit,           "Would_Exceed_Limit_Error" },
{ &MpPrivateExc_Limit_Was_Exceeded,           "Limit_Was_Exceeded_Error" },
{ &MpPrivateExc_Interface_IO_Error,           "Interface_IO_Error" },
{ &MpPrivateExc_Device_IO_Error,              "Device_IO_Error" },
{ &MpPrivateExc_File_IO_Error,                "File_IO_Error" },
{ &MpPrivateExc_Terminal_IO_Error,            "Terminal_IO_Error" },
{ &MpPrivateExc_IPC_IO_Error,                 "IPC_IO_Error" },
{ &MpPrivateExc_IPC_Connection_Lost,          "IPC_Connection_Lost_Error" },
{ &MpPrivateExc_Network_IO_Error,             "Network_IO_Error" },
{ &MpPrivateExc_Network_Connection_Lost,      "Network_Connection_Lost_Error" },
{ &MpPrivateExc_Not_Ready,                    "Not_Ready_Error" },
{ &MpPrivateExc_Interrupted,                  "Interrupted_Error" },
{ &MpPrivateExc_Pause_Requested,              "Pause_Requested_Error" },
{ &MpPrivateExc_Interface_Action_Failed,      "Interface_Action_Failed_Error" },
{ &MpPrivateExc_Device_Action_Failed,         "Device_Action_Failed_Error" },
{ &MpPrivateExc_Function_Failed,              "Function_Failed_Error" },
{ &MpPrivateExc_Controller_Internal_Error,    "Controller_Internal_Error" },
{ &MpPrivateExc_Permission_Denied,            "Permission_Denied_Error" },
{ &MpPrivateExc_Client_Request_Denied,        "Client_Request_Denied_Error" },
{ &MpPrivateExc_Try_Again,                    "Try_Again_Error" },
{ &MpPrivateExc_Timed_Out,                    "Timed_Out_Error" },
{ &MpPrivateExc_Hardware_Configuration_Error, "Hardware_Configuration_Error" },
{ &MpPrivateExc_Hardware_Fault,               "Hardware_Fault_Error" },
{ &MpPrivateExc_Record_Disabled_Due_To_Fault,
					"Record_Disabled_Due_To_Fault_Error" },
{ &MpPrivateExc_Record_Disabled_By_User,      "Record_Disabled_By_User_Error" },
{ &MpPrivateExc_Initialization_Error,         "Initialization_Error" },
{ &MpPrivateExc_Read_Only,                    "Read_Only_Error" },
{ &MpPrivateExc_Software_Configuration_Error, "Software_Configuration_Error" },
{ &MpPrivateExc_Operating_System_Error,       "Operating_System_Error" },
{ &MpPrivateExc_Unknown_Error,                "Unknown_Error_Error" },
{ &MpPrivateExc_Not_Valid_For_Current_State, 
					"Not_Valid_For_Current_State_Error" },
{ &MpPrivateExc_Configuration_Conflict,       "Configuration_Conflict_Error" },
{ &MpPrivateExc_Not_Available,                "Not_Available_Error" },
{ &MpPrivateExc_Stop_Requested,               "Stop_Requested_Error" },
{ &MpPrivateExc_Bad_Handle,                   "Bad_Handle_Error" },
{ &MpPrivateExc_Object_Abandoned,             "Object_Abandoned_Error" },
{ &MpPrivateExc_Might_Cause_Deadlock,         "Might_Cause_Deadlock_Error" },
{ &MpPrivateExc_Already_Exists,               "Already_Exists_Error" },
{ &MpPrivateExc_Invalid_Callback,             "Invalid_Callback_Error" },
{ &MpPrivateExc_Early_Exit,                   "Early_Exit_Error" },

};

static int MpPrivate_num_exceptions = sizeof( MpPrivate_exception_table )
					/ sizeof( MpPrivate_exception_table[0]);

/*---*/

/* MpErrorr_SetFromMxStatus() translates between MX errors
 * and Python exceptions.
 */

MX_EXPORT PyObject *
MpError_SetFromMxStatus( PyObject *error_object, mx_status_type mx_status )
{
	int offset;

	if ( error_object == (PyObject *) NULL ) {

		offset = mx_status.code - MXE_SUCCESS;

		if ( offset < 0 ) {
			mx_warning(
		"mx_status.code %ld has a smaller value than MXE_SUCCESS (%d)."
		"  This should not be able to happen.",
				mx_status.code, MXE_SUCCESS );

			error_object = MpPrivateExc_Error;
		} else
		if ( offset > MpPrivate_num_exceptions ) {

			error_object = MpPrivateExc_Error;
		} else {
			error_object = 
		  *(MpPrivate_exception_table[offset].exception_object_address);
		}

		if ( error_object == NULL ) {
			mx_warning(
			"Mp error object for MX error code %ld is NULL."
			"  This should not be able to happen.",
				mx_status.code );
		}
	}

	PyErr_SetString( error_object, mx_status.message );

	return NULL;
}

MX_EXPORT PyObject *
MpError_Init( PyObject *module_object )
{
	PyObject *module_dictionary;
	PyObject *new_exception;
	char *exception_name;
	char long_exception_name[80];
	int i;

	/* Find the dictionary of the module. */

	module_dictionary = PyModule_GetDict( module_object );

	if ( module_dictionary == NULL )
		return NULL;

	/* Create an Mp.Error exception class which will be the superclass
	 * for all the other exceptions.
	 */

	MpPrivateExc_Error = PyErr_NewException( "Mp.Error", NULL, NULL );

	if ( MpPrivateExc_Error == NULL )
		return NULL;

	PyDict_SetItemString( module_dictionary,
				"Error", MpPrivateExc_Error );

	/* Now create all the rest of the exceptions. */

	for ( i = 0; i < MpPrivate_num_exceptions; i++ ) {
		exception_name = MpPrivate_exception_table[i].exception_name;

		snprintf( long_exception_name, sizeof(long_exception_name),
				"Mp.%s", exception_name );

		new_exception = PyErr_NewException( long_exception_name,
					MpPrivateExc_Error, NULL );

		if ( new_exception == NULL )
			return NULL;

		*(MpPrivate_exception_table[i].exception_object_address)
			= new_exception;

		PyDict_SetItemString( module_dictionary,
					exception_name, new_exception );
	}

	return Py_None;
}

