/*
 * Name:    mp_area_detector.c
 *
 * Purpose: Python module interface to generic MX area detector functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2006-2007, 2009 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_image.h"
#include "mx_area_detector.h"
#include "mx_handle.h"
#include "mx_array.h"
#include "mp.h"

/* FIXME: Put property stuff here. */

MX_EXPORT PyObject *
MpPrivate_area_detector_get_image_format_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, format;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_image_format( record, &format );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", format );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_set_image_format_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, format;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &format );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_set_image_format( record, format );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_maximum_framesize_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, maximum_x_framesize, maximum_y_framesize;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_maximum_framesize( record,
							&maximum_x_framesize,
							&maximum_y_framesize );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "ll", maximum_x_framesize, maximum_y_framesize);

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_framesize_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, x_framesize, y_framesize;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_framesize( record,
						&x_framesize, &y_framesize );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "ll", x_framesize, y_framesize);

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_set_framesize_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_framesize";

	PyObject *framesize_tuple;
	MX_RECORD *record;
	int parse_status, tuple_size;
	long handle, x_framesize, y_framesize;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple(args, "lO", &handle, &framesize_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* The last argument is expected to be a 2-element tuple containing
	 * the X and Y framesizes.
	 */

	if ( PyTuple_Check( framesize_tuple ) == 0 ) {
		MpError_SetString( PyExc_TypeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but it is not a tuple.", cname );

		return NULL;
	}

	tuple_size = PyTuple_Size( framesize_tuple );

	if ( tuple_size != 2 ) {
		MpError_SetString( PyExc_RuntimeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but instead the tuple has %d elements.",
		cname, tuple_size );

		return NULL;
	}

	/* Get the X and Y framesizes from the tuple. */

	parse_status = PyArg_ParseTuple( framesize_tuple, "ll",
					&x_framesize, &y_framesize );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_area_detector_set_framesize( record,
						x_framesize, y_framesize );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_binsize_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, x_binsize, y_binsize;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_binsize( record,
						&x_binsize, &y_binsize );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "ll", x_binsize, y_binsize);

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_set_binsize_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_binsize";

	PyObject *binsize_tuple;
	MX_RECORD *record;
	int parse_status, tuple_size;
	long handle, x_binsize, y_binsize;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple(args, "lO", &handle, &binsize_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* The last argument is expected to be a 2-element tuple containing
	 * the X and Y binsizes.
	 */

	if ( PyTuple_Check( binsize_tuple ) == 0 ) {
		MpError_SetString( PyExc_TypeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but it is not a tuple.", cname );

		return NULL;
	}

	tuple_size = PyTuple_Size( binsize_tuple );

	if ( tuple_size != 2 ) {
		MpError_SetString( PyExc_RuntimeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but instead the tuple has %d elements.",
		cname, tuple_size );

		return NULL;
	}

	/* Get the X and Y binsizes from the tuple. */

	parse_status = PyArg_ParseTuple( binsize_tuple, "ll",
					&x_binsize, &y_binsize );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_area_detector_set_binsize( record,
						x_binsize, y_binsize );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_roi_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, roi_number;
	unsigned long roi[4];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &roi_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_roi( record,
					(unsigned long) roi_number, roi );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* The region of interest boundaries are returned
	 * as a 4-element tuple.
	 */

	result = Py_BuildValue( "ll", (long) roi[0], (long) roi[1],
					(long) roi[2], (long) roi[3] );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_set_roi_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_roi";

	PyObject *roi_tuple;
	MX_RECORD *record;
	int parse_status, tuple_size;
	long handle, roi_number;
	unsigned long roi[4];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llO",
			&handle, &roi_number, &roi_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* The last argument is expected to be a 4-element tuple containing
	 * the region of interest boundaries.
	 */

	if ( PyTuple_Check( roi_tuple ) == 0 ) {
		MpError_SetString( PyExc_TypeError,
		"The last argument to '%s' is expected to be "
		"a 4-element tuple, but it is not a tuple.", cname );

		return NULL;
	}

	tuple_size = PyTuple_Size( roi_tuple );

	if ( tuple_size != 4 ) {
		MpError_SetString( PyExc_RuntimeError,
		"The last argument to '%s' is expected to be "
		"a 4-element tuple, but instead the tuple has %d elements.",
		cname, tuple_size );

		return NULL;
	}

	/* Copy the roi boundaries in the tuple to the roi array. */

	parse_status = PyArg_ParseTuple( roi_tuple, "llll",
				&(roi[0]), &(roi[1]), &(roi[2]), &(roi[3]) );

	if ( parse_status == 0 )
		return NULL;

	mx_status = mx_area_detector_set_roi( record,
					(unsigned long) roi_number, roi );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

/* FIXME: Insert subframe or hw roi stuff here. */

MX_EXPORT PyObject *
MpPrivate_area_detector_get_bytes_per_frame_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, bytes_per_frame;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_bytes_per_frame( record,
							&bytes_per_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", bytes_per_frame );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_bytes_per_pixel_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	double bytes_per_pixel;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_bytes_per_pixel( record,
							&bytes_per_pixel );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", bytes_per_pixel );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_bits_per_pixel_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, bits_per_pixel;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_bits_per_pixel( record,
							&bits_per_pixel );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", bits_per_pixel );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_correction_flags_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	unsigned long correction_flags;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_correction_flags( record,
						&correction_flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", correction_flags );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_set_correction_flags_cmd( PyObject *self,
						PyObject *args )
{
	MX_RECORD *record;
	long handle;
	unsigned long correction_flags;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&handle, &correction_flags );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_set_correction_flags( record,
						correction_flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_measure_correction_frame_cmd( PyObject *self,
							PyObject *args )
{
	MX_RECORD *record;
	long handle, correction_measurement_type, num_correction_measurements;
	double correction_measurement_time;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lldl", &handle,
						&correction_measurement_type,
						&correction_measurement_time,
						&num_correction_measurements );
	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_measure_correction_frame( record,
						correction_measurement_type,
						correction_measurement_time,
						num_correction_measurements );
	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_use_scaled_dark_current_flag_cmd( PyObject *self,
								PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	mx_bool_type use_scaled_dark_current_flag;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_use_scaled_dark_current_flag( record,
						&use_scaled_dark_current_flag );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) use_scaled_dark_current_flag );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_set_use_scaled_dark_current_flag_cmd( PyObject *self,
								PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status, use_scaled_dark_current_flag;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "li",
				&handle, &use_scaled_dark_current_flag );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_set_use_scaled_dark_current_flag( record,
				(mx_bool_type) use_scaled_dark_current_flag );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_detector_readout_time_cmd( PyObject *self,
							PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	double detector_readout_time;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_detector_readout_time( record,
						&detector_readout_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", detector_readout_time );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_total_sequence_time_cmd( PyObject *self,
							PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	double total_sequence_time;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_total_sequence_time( record,
						&total_sequence_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", total_sequence_time );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_sequence_parameters_cmd( PyObject *self,
							PyObject *args )
{
	PyObject *result, *seq_param_tuple, *seq_item_tuple;
	MX_RECORD *record;
	MX_SEQUENCE_PARAMETERS seq;
	long handle, i;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Get the sequence parameters from MX. */

	mx_status = mx_area_detector_get_sequence_parameters( record, &seq );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to hold the contents of seq.parameter_array. */

	seq_param_tuple = PyTuple_New( seq.num_parameters );

	if ( seq_param_tuple == NULL )
		return NULL;

	/* Copy the contents of seq.parameter_array to the new tuple. */

	for ( i = 0; i < seq.num_parameters; i++ ) {

		seq_item_tuple = PyFloat_FromDouble( seq.parameter_array[i] );

		if ( seq_item_tuple == NULL )
			return NULL;
	}

	/* Create a new Python tuple containing the value of seq.sequence_type
	 * followed by the tuple 'seq_param_tuple' that we just created.
	 */

	result = Py_BuildValue( "lO", seq.sequence_type, seq_param_tuple );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_set_sequence_parameters_cmd( PyObject *self,
							PyObject *args )
{
	PyObject *seq_param_object, *seq_item_object;
	MX_RECORD *record;
	MX_SEQUENCE_PARAMETERS seq;
	long handle, i, num_parameters, sequence_type;
	int parse_status;
	mx_status_type mx_status;

	memset( &seq, 0, sizeof(MX_SEQUENCE_PARAMETERS) );

	parse_status = PyArg_ParseTuple( args, "llO",
				&handle, &sequence_type, &seq_param_object );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Copy the command arguments to an MX_SEQUENCE_PARAMETERS structure. */

	seq.sequence_type = sequence_type;

	num_parameters = PySequence_Length( seq_param_object );

	if ( num_parameters < 0 )
		return NULL;

	seq.num_parameters = num_parameters;

	for ( i = 0; i < num_parameters; i++ ) {

		seq_item_object = PySequence_GetItem( seq_param_object, i );

		if ( seq_item_object == NULL )
			return NULL;

		seq.parameter_array[i] = PyFloat_AsDouble( seq_item_object );
	}

	/* Send the sequence to MX. */

	mx_status = mx_area_detector_set_sequence_parameters( record, &seq );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_trigger_mode_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, trigger_mode;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_trigger_mode( record, &trigger_mode );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", trigger_mode );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_set_trigger_mode_cmd( PyObject *self,
						PyObject *args )
{
	MX_RECORD *record;
	long handle, trigger_mode;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
				&handle, &trigger_mode );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_set_trigger_mode( record,
				(mx_bool_type) trigger_mode );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_arm_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_arm( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_trigger_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_trigger( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_start_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_start( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_stop_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_stop( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_abort_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_abort( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_is_busy_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	mx_bool_type busy;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_is_busy( record, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) busy );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_last_frame_number_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, last_frame_number;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_last_frame_number( record,
							&last_frame_number );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", last_frame_number );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_total_num_frames_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, total_num_frames;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_total_num_frames( record,
							&total_num_frames );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", total_num_frames );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_status_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	unsigned long status_flags;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_status( record, &status_flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", status_flags );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_extended_status_cmd( PyObject *self,
						PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, last_frame_number, total_num_frames;
	unsigned long status_flags;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_extended_status( record,
			&last_frame_number, &total_num_frames, &status_flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "lll", 
		last_frame_number, total_num_frames, status_flags );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_setup_frame_cmd( PyObject *self, PyObject *args )
{
	MX_IMAGE_FRAME *image_frame;
	MX_RECORD *area_detector_record;
	long area_detector_handle, image_frame_handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
				&area_detector_handle, &image_frame_handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( area_detector_handle,
							&area_detector_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = MpPrivate_get_image_frame_from_handle( image_frame_handle,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_setup_frame( area_detector_record,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_readout_frame_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, frame_number;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &frame_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_readout_frame( record, frame_number );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_correct_frame_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_correct_frame( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_transfer_frame_cmd( PyObject *self, PyObject *args )
{
	MX_IMAGE_FRAME *image_frame;
	MX_RECORD *record;
	long handle, frame_type, image_frame_handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lll",
				&handle, &frame_type, &image_frame_handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = MpPrivate_get_image_frame_from_handle( image_frame_handle,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_transfer_frame( record,
						frame_type, image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_load_frame_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, frame_type;
	char *frame_filename;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lls",
					&handle, &frame_type, &frame_filename );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_load_frame( record,
						frame_type, frame_filename );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_save_frame_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, frame_type;
	char *frame_filename;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lls",
					&handle, &frame_type, &frame_filename );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_save_frame( record,
						frame_type, frame_filename );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_copy_frame_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, source_frame_type, destination_frame_type;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lll",
			&handle, &source_frame_type, &destination_frame_type );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_copy_frame( record,
				source_frame_type, destination_frame_type );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_area_detector_get_frame_cmd( PyObject *self, PyObject *args )
{
	MX_IMAGE_FRAME *image_frame;
	MX_RECORD *area_detector_record;
	long area_detector_handle, frame_number, image_frame_handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lll",
		&area_detector_handle, &frame_number, &image_frame_handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( area_detector_handle,
							&area_detector_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = MpPrivate_get_image_frame_from_handle( image_frame_handle,
							&image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_area_detector_get_frame( area_detector_record,
						frame_number, &image_frame );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

/* FIXME: Insert get_roi_frame here. */

