/*
 * Name:    mp_autoscale.c
 *
 * Purpose: Python module interface to generic MX autoscale functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002, 2006, 2009 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_autoscale.h"
#include "mx_handle.h"
#include "mp.h"

MX_EXPORT PyObject *
MpPrivate_autoscale_read_monitor_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, monitor_value;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_read_monitor( record, &monitor_value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", monitor_value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_get_change_request_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, change_control;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_get_change_request( record, &change_control );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", change_control );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_change_control_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, change_control;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &change_control );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_change_control( record, change_control );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_get_offset_index_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long monitor_offset_index;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_get_offset_index( record,
						&monitor_offset_index );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) monitor_offset_index );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_set_offset_index_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long monitor_offset_index;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&handle, &monitor_offset_index );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_set_offset_index( record,
				(unsigned long) monitor_offset_index );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_get_offsets_cmd( PyObject *self, PyObject *args )
{
	PyObject *offset_object, *monitor_offsets_tuple;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long i, num_monitor_offsets;
	double *monitor_offset_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_get_offset_array( record,
			&num_monitor_offsets, &monitor_offset_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to hold the returned monitor offset values. */

	monitor_offsets_tuple = PyTuple_New( (int) num_monitor_offsets );

	if ( monitor_offsets_tuple == NULL )
		return NULL;

	/* Insert the monitor offset values into the tuple. */

	for ( i = 0; i < num_monitor_offsets; i++ ) {

		offset_object = PyFloat_FromDouble( monitor_offset_array[i] );

		if ( offset_object == NULL ) {
			Py_DECREF( monitor_offsets_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( monitor_offsets_tuple, i, offset_object );
	}

	return monitor_offsets_tuple;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_set_offsets_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "set_offsets";

	PyObject *offsets_tuple, *tuple_element;
	MX_RECORD *record;
	long handle;
	int parse_status, type_status, element_status, tuple_length;
	unsigned long i, num_monitor_offsets;
	double *monitor_offset_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lO",
					&handle, &offsets_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Verify that the object we have been passed is a tuple. */

	type_status = PyTuple_Check( offsets_tuple );

	if ( type_status == 0 )
		return NULL;

	/* Find out how many objects are in the tuple. */

	tuple_length = PyTuple_Size( offsets_tuple );

	if ( tuple_length < 0 )
		return NULL;

	num_monitor_offsets = (unsigned long) tuple_length;

	/* Allocate an array to contain the monitor offset values. */

	monitor_offset_array = (double *) 
			malloc( num_monitor_offsets * sizeof(double) );

	if ( monitor_offset_array == NULL ) {
		MpError_SetString( PyExc_MemoryError,
	"%s: Unable to allocate a %lu element array of monitor offsets.",
			cname, num_monitor_offsets );
		return NULL;
	}

	/* Copy the values in offsets_tuple to monitor_offset_array. */

	for ( i = 0; i < num_monitor_offsets; i++ ) {

		/* Get the next tuple element. */

		tuple_element = PyTuple_GetItem( offsets_tuple, (int) i );

		if ( tuple_element == NULL ) {
			free( monitor_offset_array );
			return NULL;
		}

		/* Verify that the tuple element is a PyFloatObject. */

		element_status = PyFloat_Check( tuple_element );

		if ( element_status == 0 ) {
			free( monitor_offset_array );
			return NULL;
		}

		/* Copy the tuple element value to the parameter array. */

		monitor_offset_array[i] = PyFloat_AsDouble( tuple_element );
	}

	mx_status = mx_autoscale_set_offset_array( record,
						num_monitor_offsets,
						monitor_offset_array );

	free( monitor_offset_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_get_limits_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double low_limit, high_limit;
	double low_deadband, high_deadband;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_get_limits( record, &low_limit, &high_limit,
						&low_deadband, &high_deadband );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "dddd", low_limit, high_limit,
					low_deadband, high_deadband );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_set_limits_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double low_limit, high_limit;
	double low_deadband, high_deadband;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ldddd", &handle,
					&low_limit, &high_limit,
					&low_deadband, &high_deadband );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_set_limits( record, low_limit, high_limit,
						low_deadband, high_deadband );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_autoscale_compute_dynamic_limits_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double dynamic_low_limit, dynamic_high_limit;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_autoscale_compute_dynamic_limits( record,
						&dynamic_low_limit,
						&dynamic_high_limit );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "dd", dynamic_low_limit, dynamic_high_limit );

	return result;
}

