/*
 * Name:    mp_mcs.c
 *
 * Purpose: Python module interface to generic MX MCS functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002, 2006, 2009 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_mcs.h"
#include "mx_handle.h"
#include "mp.h"

MX_EXPORT PyObject *
MpPrivate_mcs_start_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_start( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mcs_stop_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_stop( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mcs_clear_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_clear( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mcs_is_busy_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_bool_type busy;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_is_busy( record, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) busy );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mcs_read_all_cmd( PyObject *self, PyObject *args )
{
	PyObject *mcs_data_tuple, *scaler_row_tuple, *measurement_object;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long i, j, num_scalers, num_measurements;
	long **mcs_data;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* MpPrivate_mcs_read_all_cmd() generates a two dimensional tuple
	 * where the rows are indexed by the scaler number and the columns
	 * are indexed by the measurement_number.
	 */

	mx_status = mx_mcs_read_all( record,
				&num_scalers, &num_measurements, &mcs_data );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create an MCS data tuple with one element per scaler. */

	mcs_data_tuple = PyTuple_New( (int) num_scalers );

	if ( mcs_data_tuple == NULL )
		return NULL;

	/* Create a scaler_row_tuple for each scaler in the mcs_data_tuple. */

	for ( i = 0; i < num_scalers; i++ ) {

		scaler_row_tuple = PyTuple_New( (int) num_measurements );

		if ( scaler_row_tuple == NULL ) {
			Py_DECREF( mcs_data_tuple );
			return NULL;
		}

		for ( j = 0; j < num_measurements; j++ ) {

			measurement_object = PyInt_FromLong( mcs_data[i][j] );

			if ( measurement_object == NULL ) {
				Py_DECREF( mcs_data_tuple );
				return NULL;
			}

			PyTuple_SET_ITEM( scaler_row_tuple, j,
							measurement_object );
		}
		PyTuple_SET_ITEM( mcs_data_tuple, i, scaler_row_tuple );
	}

	return mcs_data_tuple;
}

MX_EXPORT PyObject *
MpPrivate_mcs_read_scaler_cmd( PyObject *self, PyObject *args )
{
	PyObject *scaler_value_object, *scaler_data_tuple;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long scaler_index;
	unsigned long i, num_measurements;
	long *scaler_data;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&handle, &scaler_index );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_read_scaler( record,
					(unsigned long) scaler_index,
					&num_measurements, &scaler_data );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to hold the returned scaler_data values. */

	scaler_data_tuple = PyTuple_New( (int) num_measurements );

	if ( scaler_data_tuple == NULL )
		return NULL;

	/* Insert the scaler_data values into the tuple. */

	for ( i = 0; i < num_measurements; i++ ) {

		scaler_value_object = PyInt_FromLong( scaler_data[i] );

		if ( scaler_value_object == NULL ) {
			Py_DECREF( scaler_data_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( scaler_data_tuple, i, scaler_value_object );
	}

	return scaler_data_tuple;
}

MX_EXPORT PyObject *
MpPrivate_mcs_read_measurement_cmd( PyObject *self, PyObject *args )
{
	PyObject *measurement_value_object, *measurement_data_tuple;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long measurement_index;
	unsigned long i, num_scalers;
	long *measurement_data;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&handle, &measurement_index );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_read_measurement( record,
					(unsigned long) measurement_index,
					&num_scalers, &measurement_data );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to hold the returned measurement_data values. */

	measurement_data_tuple = PyTuple_New( (int) num_scalers );

	if ( measurement_data_tuple == NULL )
		return NULL;

	/* Insert the measurement_data values into the tuple. */

	for ( i = 0; i < num_scalers; i++ ) {

		measurement_value_object
			= PyInt_FromLong( measurement_data[i] );

		if ( measurement_value_object == NULL ) {
			Py_DECREF( measurement_data_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( measurement_data_tuple, i,
					measurement_value_object );
	}

	return measurement_data_tuple;
}

MX_EXPORT PyObject *
MpPrivate_mcs_read_timer_cmd( PyObject *self, PyObject *args )
{
	PyObject *timer_value_object, *timer_data_tuple;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long i, num_measurements;
	double *timer_data;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_read_timer( record, &num_measurements, &timer_data );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to hold the returned timer_data values. */

	timer_data_tuple = PyTuple_New( (int) num_measurements );

	if ( timer_data_tuple == NULL )
		return NULL;

	/* Insert the timer_data values into the tuple. */

	for ( i = 0; i < num_measurements; i++ ) {

		timer_value_object = PyFloat_FromDouble( timer_data[i] );

		if ( timer_value_object == NULL ) {
			Py_DECREF( timer_data_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( timer_data_tuple, i, timer_value_object );
	}

	return timer_data_tuple;
}

MX_EXPORT PyObject *
MpPrivate_mcs_get_mode_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, mode;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_get_mode( record, &mode );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", mode );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mcs_set_mode_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, mode;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &mode );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_set_mode( record, mode );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mcs_get_measurement_time_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double measurement_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_get_measurement_time( record, &measurement_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", measurement_time );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mcs_set_measurement_time_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double measurement_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld",
					&handle, &measurement_time );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_set_measurement_time( record, measurement_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mcs_get_measurement_counts_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long measurement_counts;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_get_measurement_counts( record,
						&measurement_counts );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) measurement_counts );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mcs_set_measurement_counts_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long measurement_counts;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&handle, &measurement_counts );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_set_measurement_counts( record,
					(unsigned long) measurement_counts );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mcs_get_num_measurements_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long num_measurements;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_get_num_measurements( record,
						&num_measurements );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) num_measurements );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mcs_set_num_measurements_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long num_measurements;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&handle, &num_measurements );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mcs_set_num_measurements( record,
					(unsigned long) num_measurements );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

